/// <reference path="typings/c1.mvc.basic.lib.d.ts" />
var DC101Controller = /** @class */ (function () {
    function DC101Controller(common) {
        var _this = this;
        /** グリッドID */
        this.gridId = "#results";
        /** デフォルトレイアウトのKey */
        this.defaultLayoutKey = "DC101.DefaultLayout";
        /** ユーザレイアウトのKey */
        this.userLayoutKey = "DC101.UserLayout";
        /** Excel出力のファイル名の接頭辞 */
        this.filenamePrefix = "DC101_List_";
        /** 送信ボタンのID*/
        this.submitButtonId = "#submitSearch";
        /** 日付種類選択のID*/
        this.selectorInputId = "#TgtPeriodCmb";
        /** 日付FromのID*/
        this.fromDateInputId = "#fromDate";
        /** 日付ToのID*/
        this.toDateInputId = "#toDate";
        /**
         * 全て読み完了後に実行する(c1.documentReadyにて実行)
         */
        this.documentReady = function () {
            _this.grid = wijmo.Control.getControl(_this.gridId);
            _this.common.initializeGridHeaderTooltip(_this.gridId);
            _this.common.initializeGridSelectedNone(_this.gridId);
            _this.common.initializeGridLayoutLoading(_this.gridId, _this.defaultLayoutKey, _this.userLayoutKey);
            _this.common.initializeSidePanelToggle();
            _this.common.initializeAdjustGridHeight(_this.gridId, "#search-filter");
            _this.common.initializeSubmitLoading(_this.submitButtonId);
            _this.initializeInputSetting();
        };
        /**
         * グリッドのItemFormatter
         * @param {wijmo.grid.GridPanel} panel パネル
         * @param {number} r 行INDEX
         * @param {number} c 列INDEX
         * @param {HTMLDivElement} cell Cell
         */
        this.itemFormatter = function (panel, r, c, cell) {
            //１行用のitemFormatterを適用する
            _this.common.itemFormatter(panel, r, c, cell);
            //「工事番号」でDCD01へ遷移
            // 非表示行にリンクを設定
            if (panel.columns[c].binding === "KOJI_NO" &&
                panel.cellType === wijmo.grid.CellType.Cell) {
                var url = panel.grid.getCellData(r, panel.grid.getColumn("LinkKojiNo").index, false);
                cell.innerHTML = "<a href=\"" + url + "\" onclick=\"common.linkLoadingStart(event)\">" + cell.textContent + "</a>";
            }
            // 「比率」の背景色＆フォント色制御
            if (panel.cellType === wijmo.grid.CellType.Cell && panel.columns[c].binding === "HIRITSU") {
                var ratio = panel.grid.getCellData(r, panel.grid.getColumn("HIRITSU").index, false);
                var intRatio = parseFloat(ratio);
                if (intRatio > 100.00) {
                    cell.classList.add("grid-cell-danger");
                    cell.classList.add("bg-primary");
                }
            }
        };
        this.common = common;
    }
    /** 入力コントロールの初期化 */
    DC101Controller.prototype.initializeInputSetting = function () {
        var _this = this;
        //対象期間
        var selector = wijmo.Control.getControl(this.selectorInputId);
        selector.selectedIndexChanged.addHandler(function (e) {
            var fromDate = wijmo.Control.getControl(_this.fromDateInputId);
            var toDate = wijmo.Control.getControl(_this.toDateInputId);
            var isDisabled = (e.selectedValue === 0 || e.selectedValue === 1);
            fromDate.isDisabled = isDisabled;
            toDate.isDisabled = isDisabled;
            if (isDisabled) {
                fromDate.text = '';
                toDate.text = '';
            }
            //「仕掛中含む」、「保留/中止含む」のチェックボックス
            //「完成実績」選択時のみ、選択が可能
            if (selector.selectedValue === 3) {
                $("#Filters_SIKAKARI").prop('disabled', false);
                $("#Filters_HORYU_TYUSI").prop('disabled', false);
                $("#lblSikakari").removeClass("color-gray");
                $("#lblHoryuTyusi").removeClass("color-gray");
            }
            else {
                $("#Filters_SIKAKARI").prop('checked', '');
                $("#Filters_HORYU_TYUSI").prop('checked', '');
                $("#Filters_SIKAKARI").prop('disabled', true);
                $("#Filters_HORYU_TYUSI").prop('disabled', true);
                $("#lblSikakari").addClass("color-gray");
                $("#lblHoryuTyusi").addClass("color-gray");
            }
        });
        selector.onSelectedIndexChanged();
        this.common.initializeFromToInputDate(this.fromDateInputId, this.toDateInputId);
    };
    /**
     * 検索結果のエクスポート
     */
    DC101Controller.prototype.exportExcel = function () {
        var filename = this.filenamePrefix + this.common.getNowDateTimeText() + ".xlsx";
        this.common.exportExcel(filename, this.gridId);
    };
    ;
    /** グリッドレイアウトのクリア */
    DC101Controller.prototype.clearColumnLayout = function () {
        var layout = this.common.loadColumnLayout(this.defaultLayoutKey);
        if (layout != null) {
            this.grid.columnLayout = layout;
        }
    };
    /** グリッドレイアウトの保存 */
    DC101Controller.prototype.saveColumnLayout = function () {
        this.common.saveColumnLayout(this.userLayoutKey, this.grid.columnLayout);
    };
    /** グリッドレイアウトの復元 */
    DC101Controller.prototype.loadColumnLayout = function () {
        var layout = this.common.loadColumnLayout(this.userLayoutKey);
        if (layout != null) {
            this.grid.columnLayout = layout;
        }
    };
    return DC101Controller;
}());
//commonの初期読み込み
var dc101 = new DC101Controller(common);
//conponentoneのコンポーネントが読み込まれた後
c1.documentReady(dc101.documentReady);
