﻿/// <reference path="_references.js" />



class CommonController {

    /**
     * ユーザの選択言語
     */
    public lang: UserLang.Type;

    /** Validationの後処理*/
    public postProcessValidation = () => { };
    /** body-contentを取得するセレクタ*/
    private readonly bodyContentSelector = ".body-content";

    /**
     * コンストラクタ 読み込み時に実行
     */
    public constructor() {
        this.initializeValidator();
    }

    /**
     * HTML読み込み後に実行する処理
     */
    public ready() {
        this.resizeBodyPaddingTop();
        this.initializeHistoryBack();
        this.intervalRequest();
    }

    /**
     * 定期実行
     * セッション切れを解消する為に定期的に通信を実行
     */
    public intervalRequest = () => {
        setInterval(function () {

            var url = "CostHome/Interval";

            $.ajax({
                type: "GET",
                url: url,
                success: function (result) {
                    var ret = result;
                    console.debug(ret);
                },
                error: function () {

                }
            });

        }, 60000);
    }

    /**
     * validatorの初期化
     * HTMLの描画前に実行する必要があるため、COMMONのconstructorで実行
     */
    public initializeValidator = () => {
        if ($.validator != null) {
            $.validator.unobtrusive.options = {};

            $.validator.unobtrusive.options = {
                //テキストを挿入する場合はliタグで
                errorElement: "li",
                //チェック時に実行する処理
                errorPlacement: (error: HTMLSpanElement[], inputElement: HTMLInputElement[]) => {
                    //エラーメッセージの取得、複数ある場合<br/>タグで改行して連結
                    const errorMessage = _(error)
                        .filter(x => x.innerText != null)
                        .map(x => x.innerText)
                        .reduce((x, y) => x + "<br />" + y);
                    if (errorMessage != null && errorMessage != "") {
                        //ツールチップ
                        _.each(inputElement, x => {
                            //通常のinput属性
                            let element = $(x);

                            if (x.parentElement != null && x.parentElement.classList.contains("wj-input-group")) {
                                //C1のcombobox
                                //element = $(x.parentElement).find(`[ wj-name = "${x.name}" ]`) as JQuery<HTMLInputElement>;
                                element = $(x.parentElement) as JQuery<HTMLInputElement>;
                                element.parents(".wj-control").addClass("input-validation-error");
                            }
                            element.tooltip({ html: true, placement: "auto", trigger: "click hover focus", title: errorMessage, container: "body" }); // HTMLタグ有効化でツールチップを表示
                            element.tooltip("show");

                        });
                    } else {
                        _.each(inputElement, x => {
                            //通常のinput属性
                            let element = $(x);

                            if (x.parentElement != null && x.parentElement.classList.contains("wj-input-group")) {
                                //C1のcombobox
                                element = $(x.parentElement) as JQuery<HTMLInputElement>;
                                element.parents(".wj-control").removeClass("input-validation-error");
                            }
                            $(x).tooltip("destroy");// Clear the title - there is no error associated anymore
                        });
                    }
                    //ヘッダにエラーメッセージを表示している場合を考慮し、レイアウト調整
                    $('#field-validation-area').collapse('show');
                    this.resizeBodyPaddingTop();
                    this.postProcessValidation();
                },
                success: () => {
                    //ヘッダにエラーメッセージを表示している場合を考慮し、レイアウト調整
                    this.resizeBodyPaddingTop();
                    this.postProcessValidation();
                }
            };
        }
    }

    /**
     * 現在の日時を返す 
     */
    public getNowDateTimeText(): string {
        const nowTime = new Date();
        let time = nowTime.getFullYear() + "";
        time += ("0" + (nowTime.getMonth() + 1)).slice(-2);
        time += ("0" + nowTime.getDate()).slice(-2);
        time += ("0" + nowTime.getHours()).slice(-2);
        time += ("0" + nowTime.getMinutes()).slice(-2);
        time += ("0" + nowTime.getSeconds()).slice(-2);
        return time;

    }

    /**
     * グリッドヘッダにツールチップを表示
     * @param gridId グリッドのID
     */
    public initializeGridHeaderTooltip(gridId: string) {
        const grid = wijmo.Control.getControl(gridId) as wijmo.grid.FlexGrid;

        /*ヘッダにツールチップを表示*/
        const tooltip = new wijmo.Tooltip();
        grid.hostElement.addEventListener("mouseover", (e) => {
            const ht = grid.hitTest(e);
            let rng: null | wijmo.grid.CellRange = null;
            if (ht.range != rng) {
                if (ht.cellType === wijmo.grid.CellType.ColumnHeader) {
                    rng = ht.range;
                    const data = grid.columnHeaders.getCellData(rng.row, rng.col, true);
                    const tipContent = data;
                    const cellElement = document.elementFromPoint(e.clientX, e.clientY) as Element;
                    const cellBounds = wijmo.Rect.fromBoundingRect(cellElement.getBoundingClientRect());
                    setTimeout(() => {
                        tooltip.show(grid.hostElement, tipContent, cellBounds);
                    }, FlexgridLayout.gridHeaderTooltipDelay);

                }
            }
        });
        grid.hostElement.addEventListener("mouseout", (_e) => {
            tooltip.hide();
            tooltip.dispose();
        });
    }

    /**
     * グリッドの初期表示時点で1行目選択済みなので、選択されないようにする
     * @param gridId グリッドのID
     */
    public initializeGridSelectedNone(gridId: string) {
        const grid = wijmo.Control.getControl(gridId) as wijmo.grid.FlexGrid;
        //初期表示時点で1行目選択済みなので、選択されないようにする
        grid.select(-1, -1);

    }

    /**
     * レイアウト保存状態のロード
     * @param gridId グリッドのID
     */
    public initializeGridLayoutLoading(gridId: string, defaultLayoutKey: string, userLayoutKey: string): void {
        const grid = wijmo.Control.getControl(gridId) as wijmo.grid.FlexGrid;
        //レイアウト初期状態の保存
        this.saveColumnLayout(defaultLayoutKey, grid.columnLayout);
        //レイアウト保存状態のロード
        const userLayout = this.loadColumnLayout(userLayoutKey);

        if (userLayout != null) {

            grid.columnLayout = userLayout;

            //工程を含む列が無ければ、そのまま復元
            //if (!containsKoteiColumns) {
            //    grid.columnLayout = userLayout;
            //} else {
            //    grid.columnLayout = this.getContainsKoteiColumnsLayout(grid.columnLayout, userLayout);
            //}
        }
    }

    /**
     * 工程列が存在するレイアウトの場合のColumnsLayoutを取得する
     * @param defaultLayout
     * @param userLayout
     */
    public getContainsKoteiColumnsLayout(defaultLayout: string, userLayout: string): string {
        //工程列の最大を取得
        const findKoteiRegex = /KOTEI_(CODE_|NAME_|CHECK_|)../;
        const getLayoutKoteiMax = (columns: wijmo.grid.user.IGridColumn[]) => {
            const koteiMax = _(columns)
                .filter(x => findKoteiRegex.test(x.binding))
                .map(x => Number(x.binding.slice(-2)))
                .max();
            if (koteiMax === undefined) {
                return 0;
            } else {
                return koteiMax;
            }
        }
        const defaultLayoutObj = JSON.parse(defaultLayout) as wijmo.grid.user.IGridColumnLayout;
        const userLayoutObj = JSON.parse(userLayout) as wijmo.grid.user.IGridColumnLayout;
        const defaultLayoutKoteiMax = getLayoutKoteiMax(defaultLayoutObj.columns);
        const userLayoutKoteiMax = getLayoutKoteiMax(userLayoutObj.columns);
        if (defaultLayoutKoteiMax === userLayoutKoteiMax) {
            //表示列が同じ場合、そのまま復元
            return userLayout;
        } else if (defaultLayoutKoteiMax > userLayoutKoteiMax) {
            //表示列が多い場合、最大の工程の後ろに、差分を追加して復元
            const diffLayoutColumns = _.filter(defaultLayoutObj.columns, x => findKoteiRegex.test(x.binding) && Number(x.binding.slice(-2)) > userLayoutKoteiMax);
            const insertTagetIndex = _.findLastIndex(userLayoutObj.columns, x => findKoteiRegex.test(x.binding) && Number(x.binding.slice(-2)) === userLayoutKoteiMax) + 1;
            userLayoutObj.columns.splice(insertTagetIndex, 0, ...diffLayoutColumns);
            return JSON.stringify(userLayoutObj);
        } else {
            //表示列が少ない場合、差分を削除して復元
            _.remove(userLayoutObj.columns, x => findKoteiRegex.test(x.binding) && Number(x.binding.slice(-2)) > defaultLayoutKoteiMax);
            return JSON.stringify(userLayoutObj);
        }
    }

    /**
     * グリッドのItemFormatter
     * @param {wijmo.grid.GridPanel} panel パネル
     * @param {number} r 行INDEX
     * @param {number} _c 列INDEX
     * @param {HTMLDivElement} cell Cell
     */
    public itemFormatter = (panel: wijmo.grid.GridPanel, r: number, _c: number, cell: HTMLDivElement): void => {

        //ヘッダー高さ調節・改行有効
        if (panel.cellType === wijmo.grid.CellType.ColumnHeader) {
            //cell.classList.add("grid-header-base");
            //ヘッダは左寄せ
            cell.style.textAlign = "left";
            //panel.rows[r].height = FlexgridLayout.HeaderHeight;
        } else if (panel.cellType === wijmo.grid.CellType.Cell) {
            panel.rows[r].height = FlexgridLayout.RowHeight;
        }


    }

    /**
     * グリッドのItemFormatterDouble(２行用)
     * @param {wijmo.grid.GridPanel} panel パネル
     * @param {number} r 行INDEX
     * @param {number} _c 列INDEX
     * @param {HTMLDivElement} cell Cell
     */
    public itemFormatterDouble = (panel: wijmo.grid.GridPanel, r: number, _c: number, cell: HTMLDivElement): void => {

        //ヘッダー高さ調節・改行有効
        if (panel.cellType === wijmo.grid.CellType.ColumnHeader) {
            cell.classList.add("grid-header-base");
            //ヘッダは左寄せ
            cell.style.textAlign = "left";
            panel.rows[r].height = FlexgridLayout.HeaderHeightDouble;
        } else if (panel.cellType === wijmo.grid.CellType.Cell) {
            panel.rows[r].height = FlexgridLayout.RowHeight;
        }


    }

    /**
     * グリッドの計画日と実績日の背景色＆フォント色制御
     * @param {wijmo.grid.GridPanel} panel パネル
     * @param {number} r 行INDEX
     * @param {number} _c 列INDEX
     * @param {HTMLDivElement} cell Cell
     */
    public changeCellBackFont = (panel: wijmo.grid.GridPanel, r: number, _c: number, cell: HTMLDivElement, yotei: string, jisseki: string): void => {

        if (panel.columns[_c].binding === jisseki &&
            panel.cellType === wijmo.grid.CellType.Cell) {
            const sysDate = new Date();
            const yoteiString = panel.grid.getCellData(r, panel.grid.getColumn(yotei).index, false) as string;
            const yoteiDate = panel.grid.getCellData(r, panel.grid.getColumn(yotei).index, false) as Date;
            const jissekiString = panel.grid.getCellData(r, panel.grid.getColumn(jisseki).index, false) as string;
            const jissekiDate = panel.grid.getCellData(r, panel.grid.getColumn(jisseki).index, false) as Date;
            if (yoteiString === "" || yoteiString === null) {
                //予定日がない場合は、制御なし
            } else {
                //予定日がある場合
                if (jissekiString === "" || jissekiString === null) {
                    //実績日がなくて、予定日から遅れていれば、背景色を赤へ
                    if (yoteiDate < sysDate) {
                        cell.classList.add("grid-cell-danger");
                    }
                } else {
                    //実績日があって、予定日から遅れていれば、フォント色を赤へ
                    if (yoteiDate < jissekiDate) {
                        cell.classList.add("text-danger");
                    }
                }
            }
        }
    }

    /**
     * グリッドの計画日と実績日の背景色＆フォント色制御（グレーアウト）
     * @param {wijmo.grid.GridPanel} panel パネル
     * @param {number} r 行INDEX
     * @param {number} _c 列INDEX
     * @param {HTMLDivElement} cell Cell
     */
    public changeCellBackFontGrayOut = (panel: wijmo.grid.GridPanel, r: number, _c: number, cell: HTMLDivElement, yotei: string, jisseki: string): void => {

        //計画日
        if (panel.columns[_c].binding === yotei &&
            panel.cellType === wijmo.grid.CellType.Cell) {
            const yoteiString = panel.grid.getCellData(r, panel.grid.getColumn(yotei).index, false) as string;
            const jissekiString = panel.grid.getCellData(r, panel.grid.getColumn(jisseki).index, false) as string;
            //予定日がない
            if (yoteiString === "" || yoteiString === null) {
                //実績日がない
                if (jissekiString === "" || jissekiString === null) {
                    cell.classList.add("grid-cell-none");
                }
            }
        }
        //実績日
        if (panel.columns[_c].binding === jisseki &&
            panel.cellType === wijmo.grid.CellType.Cell) {
            const yoteiString = panel.grid.getCellData(r, panel.grid.getColumn(yotei).index, false) as string;
            const jissekiString = panel.grid.getCellData(r, panel.grid.getColumn(jisseki).index, false) as string;
            //予定日がない
            if (yoteiString === "" || yoteiString === null) {
                //実績日がない
                if (jissekiString === "" || jissekiString === null) {
                    cell.classList.add("grid-cell-none");
                }
            }
        }
    }

    /**
     * ヘッダに合わせ、Bodyを下げる
     */
    public resizeBodyPaddingTop() {
        // navbarの高さを取得する
        const height = $('.navbar').innerHeight();
        if (height != null) {
            // bodyのpaddingにnavbarの高さを設定する
            $(this.bodyContentSelector).css('padding-top', height);

        }
    }

    /**
     * チェックボックスとフィルタ処理を対応させる初期設定
     * @param gridId グリッドのID　＃をつける
     * @param gridFilterId グリッドのフィルタのID　＃をつけない
     * @param checkboxId
     * @param columnName
     * @param filterKey
     */
    public setFilterAction(gridId: string, gridFilterId: string, checkboxId: string, columnName: string) {
        $(checkboxId).change(() => {
            this.filterAction(gridId, gridFilterId, checkboxId, columnName);
        });
    };
    /**
     * チェックボックスとフィルタ処理を対応させる
     * @param gridId
     * @param gridFilterId
     * @param checkboxId
     * @param columnName
     * @param filterKey
     */
    public filterAction(gridId: string, gridFilterId: string, checkboxId: string, columnName: string) {
        const checked = $(checkboxId).prop("checked");
        const value = $(checkboxId).val() as string;
        // ■値フィルタによるフィルタリング
        const grid = wijmo.Control.getControl(gridId) as wijmo.grid.FlexGrid;
        // フィルタの取得
        const filter = c1.getExtender(grid, gridFilterId) as wijmo.grid.filter.FlexGridFilter;
        // カラムフィルタの取得（5列目のフィルタ）
        const columnFilter = filter.getColumnFilter(columnName);
        if (checked) {
            // 抽出条件の指定
            if (columnFilter.valueFilter.showValues != null) {
                columnFilter.valueFilter.showValues[value] = true;
            } else {
                const valueFilter = { [value]: true };
                columnFilter.valueFilter.showValues = valueFilter;
            }

        } else {
            // 抽出条件の指定
            if (columnFilter.valueFilter.showValues != null) {
                delete columnFilter.valueFilter.showValues[value];
            } else {
                columnFilter.valueFilter.clear();
            }
        }
        // フィルタの適用
        filter.apply();
    }

    /**
     * Export Excel のオプション
     */
    public flexGridXlsxOptions: wijmo.grid.xlsx.IFlexGridXlsxOptions = {
        includeCellStyles: true,
        includeColumnHeaders: true,
        includeColumns: (column: wijmo.grid.Column) => column.isVisible,
        formatItem: (args: wijmo.grid.xlsx.XlsxFormatItemEventArgs) => {
            if ((args.xlsxCell.style != null) && (args.panel.columns[args.col].dataType == wijmo.DataType.String)) {
                (args.xlsxCell.style as wijmo.xlsx.IWorkbookStyle).format = "@";
            }
        }
    };


    /**
     * 検索結果のエクスポート
     */
    public exportExcel(filename: string, gridId: string) {
        this.loadingStart();
        const grid = wijmo.Control.getControl(gridId) as wijmo.grid.FlexGrid;

        //2020/01/06 T.Sasaki ページング対応
        var cv = grid.collectionView as wijmo.collections.CollectionView;
        var pageSize = cv.pageSize;
        cv.pageSize = 0;

        //非同期で実行
        setTimeout(() => {
            wijmo.grid.xlsx.FlexGridXlsxConverter.save(
                grid,
                this.flexGridXlsxOptions,
                filename);
            //this.loadingEnd();
        }, this.loadingEndTransitionDuration);

        //2020/01/06 T.Sasaki ページング対応
        setTimeout(() => {
            cv.pageSize = pageSize;
            this.loadingEnd();
        }, this.loadingEndTransitionDuration);

    };

    /**
     * loadingアイコンのPromise
     */
    public loadingPromises?: JQuery.Promise<any, any, any>[];

    /** Loadingの開始の遅延時間(ms)*/
    private readonly loadingStartTransitionDuration = 3000;

    /**
     * Loadingの開始。loadingEndするまで画面のクリック操作ができなくなる
     */
    public loadingStart() {
        $("#loader").removeClass("is-hide");
        const promise = $(".loading").css({ opacity: "0" }).animate({ opacity: "100" }, this.loadingStartTransitionDuration).promise();
        if (this.loadingPromises == null)
            this.loadingPromises = [promise];
        else
            this.loadingPromises.push(promise);
    }

    /** Loadingの終了の遅延時間(ms)*/
    private readonly loadingEndTransitionDuration = 500;

    /**
     * Loadingの終了。loadingEndするまで画面のクリック操作ができなくなる
     */
    public loadingEnd() {
        if (this.loadingPromises == null)
            $("#loader").addClass("is-hide");
        else
            $.when(this.loadingPromises).done(() => {
                $(".loading").css({ opacity: "100" }).animate({ opacity: "0" }, this.loadingEndTransitionDuration, () => {
                    $("#loader").addClass("is-hide");
                });
            });
    }


    /** 
     *  画面遷移時のLoading表示。別タブ/ウィンドウで開くクリックの場合、loadingを表示しない。
     * @description ロジックはMediaWikiから流用
     */
    public linkLoadingStart(event?: JQuery.Event<HTMLElement, null>) {
        if (event == null) return;
        const button = event.button;
        // Add which for click: 1 === left; 2 === middle; 3 === right
        // Note: button is not normalized, so don't use it
        if (!event.which && button !== undefined) {
            event.which = (button & 1 ? 1 : (button & 2 ? 3 : (button & 4 ? 2 : 0)));
        }
        /*
         * 別タブ/ウィンドウで開くクリックの判定箇所
         * https://github.com/wikimedia/mediawiki-extensions-MultimediaViewer/blob/6b83fca574e3dcc2fa4526117f88940dcfc06f30/resources/mmv/mmv.bootstrap.js#L431-L434
         */
        if ((event.button !== 0 && event.which !== 1) || event.altKey || event.ctrlKey || event.shiftKey || event.metaKey) {
            // 別タブ/ウィンドウで開くクリック
        } else {
            // 通常のクリック
            this.loadingStart();
        }
    }

    /**
     * サイドパネルを初期化
     * @param width サイドパネルの幅
     */
    public initializeSidePanelToggle() {
        /* Set the width of the side navigation to (width) */
        $(".sidepanel-toggle").click((_e) => {
            $('body').toggleClass('side-open');
        })
        /* Set the width of the side navigation to 0 */
        $(".sidepanel").click((_e) => {
            $('body').toggleClass('side-open');
        })
    }


    /**
     * グリッドレイアウトの保存
     */
    public saveColumnLayout(keyName: string, columnLayout: string): void {
        localStorage.setItem(keyName, columnLayout);
    };
    /**
     * グリッドレイアウトの読み込み
     */
    public loadColumnLayout(keyName: string): string | null {
        const layout = localStorage.getItem(keyName);
        if (layout != null) return layout;
        else return null;
    };

    /**
     * グリッドの高さをウインドウに合わせる
     * @param gridId グリッドID
     * @param anotherElements グリッド以外の要素のID(ヘッダ、フッタは除く)
     */
    public adjustGridHeight(gridId: string, ...anotherElements: string[]) {
        const margin = 10;
        const windowHeight = window.innerHeight;
        const body = $(this.bodyContentSelector);
        const windowMargin = parseInt(body.css("padding-top")) + parseInt(body.css("padding-bottom"));
        const body_h = windowHeight - windowMargin;

        //パネルのサイズ調節
        const anotherHeight = _(anotherElements).map(x => $(x).outerHeight(true) as number).sum();
        //パネル内の表のサイズ調節
        $(gridId)[0].style.height = body_h - anotherHeight - margin + "px";

        wijmo.Control.invalidateAll();

    }
    /**
     * 初期設定グリッドの高さをウインドウに合わせる 
     * @param gridId グリッドID
     * @param anotherElements グリッド以外の要素のID(ヘッダ、フッタは除く)
     */
    public initializeAdjustGridHeight(gridId: string, ...anotherElements: string[]) {
        this.adjustGridHeight(gridId, ...anotherElements);
        $(window).on("load resize", () => {
            this.adjustGridHeight(gridId, ...anotherElements);
        });
    }

    /** ボタン押下時のLoading表示の初期設定 */
    public initializeSubmitLoading(submitButtonId: string | null = null, ...LinkIds: string[]): void {
        if (submitButtonId != null) {
            $(submitButtonId).click((e) => {
                //クライアント入力チェックでOKの場合のみ
                if ($("#mainForm").valid()) {
                    this.linkLoadingStart(e);
                }
            });

        }

        _.each(LinkIds, x => $(x).click((e) => { this.linkLoadingStart(e); }));

    }

    /**
     * 日付入力の範囲指定で、Fromの場合Toが最大/Toの場合Fromが最小になる設定を実施
     * @param fromDateId Fromの日付入力のId(#付き)
     * @param toDateId Toの日付入力のId(#付き)
     */
    public initializeFromToInputDate(fromDateId: string, toDateId: string) {
        const fromDate = wijmo.Control.getControl(fromDateId) as wijmo.input.InputDate;
        const toDate = wijmo.Control.getControl(toDateId) as wijmo.input.InputDate;
        fromDate.valueChanged.addHandler((e: wijmo.input.InputDate) => {
            toDate.min = e.value != null ? e.value : new Date(1900, 0, 1);
        });
        toDate.valueChanged.addHandler((e: wijmo.input.InputDate) => {
            fromDate.max = e.value != null ? e.value : new Date(2100, 0, 1);
        });
        fromDate.onValueChanged();
        toDate.onValueChanged();

    }
    /** 
     *  戻るボタン #historyback を初期化 / 
     * */
    public initializeHistoryBack() {
        let bs = false; //history.backを実行後にページ移動が発生したかどうか
        //unload beforeunloadは「戻る」が押されたときに走るイベント
        $(window).bind("unload beforeunload", () => {
            bs = true;
        });
        $('#historyback').on("click", (e: JQuery.Event) => {
            const href = <string>$(e.target).attr("href");
            history.back();
            setTimeout(() => {
                if (!bs) {
                    //新しいウィンドウで開いた場合など「戻る」が使えない場合はリファラーに移動する
                    location.href = href;
                }
            }, 100);
            return false;
        });
    }

    /**
     * viewに隠しで持っている値を取得する
     */
    public getHiddenValue = (name: string): string => $(':hidden[name="' + name + '"]').val() as string;

    /**
     * グリッドの高さをタブパネルに合わせる
     * @param gridId グリッドID
     * @param anotherElements グリッド以外の要素のID(ヘッダ、フッタは除く)
     */
    public adjustHeightGridInTab(gridId: string, tabPanelId: string, ...anotherElements: string[]) {
        const margin = 10;
        const body = $(this.bodyContentSelector);
        const paddingBottom = parseInt(body.css("padding-bottom"));//フッタの高さ
        var tabpanel = wijmo.Control.getControl(tabPanelId) as wijmo.nav.TabPanel;
        var selectedTab = tabpanel.selectedTab as wijmo.nav.Tab;//選択中のタブ   
        var tabPaneHeight = selectedTab.pane.clientHeight;//タブ内容領域の高さ

        var body_h = tabPaneHeight - paddingBottom;

        //パネルのサイズ調節
        const anotherHeight = _(anotherElements).map(x => $(x).outerHeight(true) as number).sum();
        //パネル内の表のサイズ調節
        $(gridId)[0].style.height = body_h - anotherHeight - margin + "px";

        wijmo.Control.invalidateAll();

    }

    /**
     * 初期設定グリッドの高さをウインドウに合わせる 
     * @param gridId グリッドID
     * @param anotherElements グリッド以外の要素のID(ヘッダ、フッタは除く)
     */
    public initializeAdjustHeightGridInTab(gridId: string, tabPanelId: string, ...anotherElements: string[]) {
        this.adjustHeightGridInTab(gridId, tabPanelId, ...anotherElements);
        $(window).on("load resize", () => {
            this.adjustHeightGridInTab(gridId, tabPanelId, ...anotherElements);
        });
    }

    /**
     * タブパネルの高さをウィンドウに合わせる
     * @param gridId グリッドID
     * @param anotherElements グリッド以外の要素のID(ヘッダ、フッタは除く)
     */
    public adjustHeightTabPanel(tabPanelId: string, ...anotherElements: string[]) {
        const margin = 10;
        const windowHeight = window.innerHeight;
        const body = $(this.bodyContentSelector);
        const windowMargin = parseInt(body.css("padding-top")) + parseInt(body.css("padding-bottom"));
        const body_h = windowHeight - windowMargin;
        var tabpanel = wijmo.Control.getControl(tabPanelId) as wijmo.nav.TabPanel;
        var selectedTab = tabpanel.selectedTab as wijmo.nav.Tab
        var tabHeaderHeight = selectedTab.header.clientHeight;

        //その他コントロールのサイズ
        const anotherHeight = _(anotherElements).map(x => $(x).outerHeight(true) as number).sum();
        //パネル(Pane)のサイズ調節
        selectedTab.pane.style.height = body_h - anotherHeight - tabHeaderHeight - margin + "px";

        wijmo.Control.invalidateAll();

    }

    /**
     * 初期設定タブパネルの高さをウインドウに合わせる
     * @param gridId グリッドID
     * @param anotherElements グリッド以外の要素のID(ヘッダ、フッタは除く)
     */
    public initializeAdjustHeightTabPanel(tabPanelId: string, ...anotherElements: string[]) {
        this.adjustHeightTabPanel(tabPanelId, ...anotherElements);
        $(window).on("load resize", () => {
            this.adjustHeightTabPanel(tabPanelId, ...anotherElements);
        });
    }

}
//commonの初期読み込み
const common = new CommonController;


//HTML読み込み後に実行する処理
$(() => {
    common.ready();
});